@@ -835,8 +835,8 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 			_vte_terminal_home_cursor (terminal);
 		}
 		/* Reset scrollbars and repaint everything. */
-		terminal->adjustment->value =
-			terminal->pvt->screen->scroll_delta;
+		gtk_adjustment_set_value(terminal->adjustment,
+					 terminal->pvt->screen->scroll_delta);
 		vte_terminal_set_scrollback_lines(terminal,
 				terminal->pvt->scrollback_lines);
 		_vte_terminal_queue_contents_changed(terminal);
@@ -975,20 +975,15 @@ vte_sequence_handler_bt (VteTerminal *terminal, GValueArray *params)
 
 	if (terminal->pvt->tabstops != NULL) {
 		/* Find the next tabstop. */
-		while (newcol >= 0) {
+		while (newcol > 0) {
+			newcol--;
 			if (_vte_terminal_get_tabstop(terminal,
 						     newcol % terminal->column_count)) {
 				break;
 			}
-			newcol--;
 		}
 	}
 
-	/* If we have no tab stops, stop at the first column. */
-	if (newcol <= 0) {
-		newcol = 0;
-	}
-
 	/* Warp the cursor. */
 	_vte_debug_print(VTE_DEBUG_PARSE,
 			"Moving cursor to column %ld.\n", (long)newcol);
@@ -2344,11 +2339,8 @@ vte_sequence_handler_character_attributes (VteTerminal *terminal, GValueArray *p
 			break;
 		}
 		case 39:
-			/* default foreground, no underscore */
+			/* default foreground */
 			terminal->pvt->screen->defaults.attr.fore = VTE_DEF_FG;
-			/* By ECMA 48, this underline off has no business
-			   being here, but the Linux console specifies it. */
-			terminal->pvt->screen->defaults.attr.underline = 0;
 			break;
 		case 40:
 		case 41:
@@ -3041,6 +3033,7 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 	long param, arg1, arg2;
 	gint width, height;
 	guint i;
+	GtkAllocation allocation;
 
 	widget = &terminal->widget;
 	screen = terminal->pvt->screen;
@@ -3152,16 +3145,16 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 			/* If we're unmapped, then we're iconified. */
 			g_snprintf(buf, sizeof(buf),
 				   _VTE_CAP_CSI "%dt",
-				   1 + !GTK_WIDGET_MAPPED(widget));
+				   1 + !gtk_widget_get_mapped(widget));
 			_vte_debug_print(VTE_DEBUG_PARSE,
 					"Reporting window state %s.\n",
-					GTK_WIDGET_MAPPED(widget) ?
+					gtk_widget_get_mapped(widget) ?
 					"non-iconified" : "iconified");
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 13:
 			/* Send window location, in pixels. */
-			gdk_window_get_origin(widget->window,
+			gdk_window_get_origin(gtk_widget_get_window(widget),
 					      &width, &height);
 			g_snprintf(buf, sizeof(buf),
 				   _VTE_CAP_CSI "3;%d;%dt",
@@ -3175,12 +3168,13 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 			break;
 		case 14:
 			/* Send window size, in pixels. */
+			gtk_widget_get_allocation(widget, &allocation);
 			g_snprintf(buf, sizeof(buf),
 				   _VTE_CAP_CSI "4;%d;%dt",
-				   widget->allocation.height -
+				   allocation.height -
                                        (terminal->pvt->inner_border.top +
                                         terminal->pvt->inner_border.bottom),
-				   widget->allocation.width -
+				   allocation.width -
                                        (terminal->pvt->inner_border.left +
                                         terminal->pvt->inner_border.right));
 			_vte_debug_print(VTE_DEBUG_PARSE,
@@ -3213,21 +3207,29 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 20:
-			/* Report the icon title. */
+			/* Report a static icon title, since the real
+			   icon title should NEVER be reported, as it
+			   creates a security vulnerability.  See
+			   http://marc.info/?l=bugtraq&m=104612710031920&w=2
+			   and CVE-2003-0070. */
 			_vte_debug_print(VTE_DEBUG_PARSE,
-				"Reporting icon title.\n");
+				"Reporting fake icon title.\n");
+			/* never use terminal->icon_title here! */
 			g_snprintf (buf, sizeof (buf),
-				    _VTE_CAP_OSC "L%s" _VTE_CAP_ST,
-				    terminal->icon_title);
+				    _VTE_CAP_OSC "LTerminal" _VTE_CAP_ST);
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 21:
-			/* Report the window title. */
+			/* Report a static window title, since the real
+			   window title should NEVER be reported, as it
+			   creates a security vulnerability.  See
+			   http://marc.info/?l=bugtraq&m=104612710031920&w=2
+			   and CVE-2003-0070. */
 			_vte_debug_print(VTE_DEBUG_PARSE,
-					"Reporting window title.\n");
+					"Reporting fake window title.\n");
+			/* never use terminal->window_title here! */
 			g_snprintf (buf, sizeof (buf),
-				    _VTE_CAP_OSC "l%s" _VTE_CAP_ST,
-				    terminal->window_title);
+				    _VTE_CAP_OSC "lTerminal" _VTE_CAP_ST);
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		default:
